<?php

/*
 *  location: catalog/extension/payment
 */

require_once DIR_SYSTEM . 'library/make_commerce/maksekeskus/vendor/autoload.php';

use Maksekeskus\Maksekeskus;

class ControllerExtensionPaymentMakeCommerce extends Controller {

    /**
     * Instance of the Maksekeskus SDK class
     */
    private $api;

    /**
     * Module settings
     */
    private $settings;

    /**
     * Payment methods settings
     */
    private $payment_methods;

    /**
     * Is test environment
     */
    private $is_test_env;

    /**
     * Payment gateway url
     */
    private $static_gateway_url;

    /**
     * Loads settings and sets up properties.
     *
     * @param  array $registry
     * @return void
     */
    public function __construct($registry)
    {
        parent::__construct($registry);

        $this->config->load('make_commerce');
        $this->load->model('extension/payment/make_commerce');

        $this->payment_methods = $this->model_extension_payment_make_commerce->getPaymentMethods();
        $this->settings        = $this->model_extension_payment_make_commerce->getSettings();
        $this->is_test_env     = $this->settings['api_environment'] !== 'live';

        $shop_id     = ($this->is_test_env) ? $this->settings['shop_id_test'] : $this->settings['shop_id_live'];
        $public_key  = ($this->is_test_env) ? $this->settings['public_key_test'] : $this->settings['public_key_live'];
        $private_key = ($this->is_test_env) ? $this->settings['private_key_test'] : $this->settings['private_key_live'];

        $this->api = new Maksekeskus($shop_id, $public_key, $private_key, $this->is_test_env);

        if ($this->is_test_env)
            $this->static_gateway_url = $this->config->get('mk_test_static_gateway_url');
        else
            $this->static_gateway_url = $this->config->get('mk_live_static_gateway_url');

    }

    /**
     * Main method of the controller.
     *
     * @return string
     */
    public function index()
    {
        $this->language->load('extension/payment/make_commerce');
        $this->load->model('checkout/order');

        $this->document->addStyle('catalog/view/javascript/make_commerce/stylesheet/make_commerce.css');

        $make_commerce_selected_method = (empty($this->session->data['make_commerce_method'])) ? '' : $this->session->data['make_commerce_method'];
        $action                        = 'index.php?route=extension/payment/make_commerce/processPayment';

        $make_commerce_method         = explode('_', $make_commerce_selected_method);
        $make_commerce_method_country = (count($make_commerce_method) > 1) ? $make_commerce_method[0] : '';
        $make_commerce_method_name    = (count($make_commerce_method) > 1) ? $make_commerce_method[1] : '';
        $script_src                   = htmlspecialchars($this->static_gateway_url . 'checkout/dist/checkout.js');
        $is_card                      = false;
        $js_params                    = '';
        $return_url_cc                = '';
        $isCurrentUrlHttps            = $this->isCurrentUrlHttps();

        foreach ($this->payment_methods['cards'] as $card)
            if ($card['name'] === $make_commerce_method_name)
                $is_card = true;

        if ($is_card) {
            $order_info  = $this->model_checkout_order->getOrder($this->session->data['order_id']);
            $order_total = $order_info['total'];
            $currency    = $this->session->data['currency'];
            $action      = '';

            $order_id               = $this->session->data['order_id'];
            $customer_ip            = $order_info['ip'];
            $order_total            = $this->currency->convert($order_info['total'], $currency, 'EUR');
            $public_key             = ($this->is_test_env) ? $this->settings['public_key_test'] : $this->settings['public_key_live'];
            $customer_language_code = $order_info['language_code'];

            $billing_country = strtolower($order_info['payment_iso_code_2']);
            $return_url      = ($isCurrentUrlHttps ? HTTPS_SERVER : HTTP_SERVER) . 'index.php?route=extension/payment/make_commerce/callback';
            $amount          = number_format((float)$order_total, 2, '.', '');
            $locale          = strtolower(substr($customer_language_code, 0, 2));
            if ($locale=="") $locale = "en";  // english GUI if locale is not set because of some config issue

            $transaction       = $this->createTransaction($amount, $order_id, $return_url, $customer_ip, $billing_country, $locale);
            $payment_firstname = $order_info['payment_firstname'];
            $payment_lastname  = $order_info['payment_lastname'];
            $customer_email    = $order_info['email'];
            $return_url_cc     = $return_url;

            $this->model_extension_payment_make_commerce->updateOrderData($order_id, 'transaction_id', $transaction->id);

            $data['transaction_id'] = $transaction->id;

            $js_params = array(
                'key'          => $public_key,
                'transaction'  => $transaction->id,
                'selector'     => '#mk-pay-button',
                'amount'       => $amount,
                'locale'       => $locale,
                'open-on-load' => 'false',
                'client-name'  => ($this->settings['prefill_credit_card_form'] == 'on' ? (string) ($payment_firstname . ' ' . $payment_lastname) : ''),
                'email'        => ($this->settings['prefill_credit_card_form'] == 'on' ? (string) $customer_email : ''),
                'name'         => $this->settings['shop_name'],
                'description'  => 'Order &nbsp;' . (string) $order_id,
                'completed'    => 'makecommerce_cc_complete',
                'currency'     => 'EUR',
                'no_conflict'  => 'true'
            );

            $js_params = $this->convertArrayToHtmlAttributes($js_params);
        }

        $data['is_card']              = $is_card;
        $data['make_commerce_method'] = $make_commerce_selected_method;
        $data['payment_not_selected'] = $this->language->get('payment_not_selected');
        $data['button_pay']           = $this->language->get('button_pay');
        $data['button_confirm']       = $this->language->get('button_confirm');
        $data['action']               = $action;
        $data['script_src']           = $script_src;
        $data['js_params']            = $js_params;
        $data['return_url_cc']        = $return_url_cc;

        return $this->load->view('extension/payment/make_commerce', $data);
    }

    /**
     * Sets seleted MakeCommerce option.
     *
     * @return void
     */
    public function set()
    {
        $selected_method = $this->request->post['selected_method'];
        $lang            = (isset($this->session->data['language'])) ? $this->session->data['language'] : $this->config->get('config_language');
        $method_names    = $this->settings['payments_widget_language_title'];
        $title           = (isset($method_names[$lang]) ? $method_names[$lang] : '');

        $this->session->data['make_commerce_method'] = $selected_method;

        if (isset($this->session->data['payment_methods']) && array_key_exists('make_commerce', $this->session->data['payment_methods']))
            $this->session->data['payment_methods']['make_commerce']['title'] = $title;

        print(200);
    }

    /**
     * Creates a transaction and redirects the user to MakeCommerce payment page accordingly.
     *
     * @return void
     */
    public function processPayment()
    {
        $this->load->model('checkout/order');

        $order_info           = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        $make_commerce_method = $this->session->data['make_commerce_method'];
        $currency             = $this->session->data['currency'];
        $make_commerce_method = explode('_', $make_commerce_method);
        $isCurrentUrlHttps    = $this->isCurrentUrlHttps();

        $order_id               = $this->session->data['order_id'];
        $customer_ip            = $order_info['ip'];
        $order_total            = $order_info['total'];
        $customer_language_code = strtolower($order_info['language_code']);

        $billing_country = $order_info['payment_iso_code_2'];
        $return_url      = ($isCurrentUrlHttps ? HTTPS_SERVER : HTTP_SERVER) . 'index.php?route=extension/payment/make_commerce/callback';
        $amount          = number_format((float)$order_total, 2, '.', '');
        $locale          = strtolower(substr($customer_language_code, 0, 2));
        if ($locale=="") $locale = "en";  // english GUI if locale is not set because of some config issue

        $transaction  = $this->createTransaction($amount, $order_id, $return_url, $customer_ip, $billing_country, $locale);
        $redirect_url = $this->getPaymentMethodUrlByName($transaction, $this->payment_methods['banklinks'], $make_commerce_method[0], $make_commerce_method[1]);

        $this->model_extension_payment_make_commerce->updateOrderData($order_id, 'transaction_id', $transaction->id);

        $this->response->redirect($redirect_url);
    }

    function curl_loopback( $url, $data ) {
        $curl = curl_init();
        if (isset($_SERVER['HTTPS'])) { if ($_SERVER['HTTPS']=="on") $isCurrentUrlHttps = 1; else $isCurrentUrlHttps = 0; } else $isCurrentUrlHttps = 0;
        if ($isCurrentUrlHttps) {
            curl_setopt( $curl, CURLOPT_PORT, 443);
        }
        curl_setopt( $curl, CURLOPT_HEADER, false);
        curl_setopt( $curl, CURLOPT_HTTPHEADER, array("Cookie: OCSESSID=".$data["OCSESSID"]));
        curl_setopt( $curl, CURLINFO_HEADER_OUT, true);
        curl_setopt( $curl, CURLOPT_USERAGENT, $this->request->server['HTTP_USER_AGENT']);
        curl_setopt( $curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt( $curl, CURLOPT_FORBID_REUSE, false);
        curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt( $curl, CURLOPT_URL, $url);
        curl_setopt( $curl, CURLOPT_POST, true);
        curl_setopt( $curl, CURLOPT_POSTFIELDS, $data );
        $json = curl_exec( $curl );
        if (!$json) {
            $response["error"] = curl_error($curl);
            $response["errno"] = curl_errno($curl);
            $json = json_encode($response);
        }
        curl_close($curl);
        return $json;
    }

    /**
     * Payment callback method.
     *
     * @return void
     */
    public function callback()
    {
        if ($_SERVER['REQUEST_METHOD'] !== 'POST')
            die('Illegal Access');

        $this->load->model('checkout/order');
        $this->load->model('extension/payment/make_commerce');

        // detect cartToOrder incoming request
        if ( count($_POST) == 0) {
            $jsn       = file_get_contents('php://input');
            $orderInfo = json_decode($jsn);
            $this->load->model('extension/module/make_commerce_sco');
            $postdata["OCSESSID"] = $this->model_extension_module_make_commerce_sco->getCartSessionId($orderInfo->cartId);
            $postdata["json"] = $jsn;
            $url = ((isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] == "on" || $_SERVER["HTTPS"] == 1)) ? "https://" : "http://" ) .
                    $_SERVER["SERVER_NAME"] . $_SERVER["SCRIPT_NAME"] . "?route=extension/checkout/mk_cart_to_order";
            $result = $this->curl_loopback( $url , $postdata);
            echo $result;
            die();
        }

        $request      = $this->stripslashesDeep($_POST);
        $lang         = (isset($this->session->data['language'])) ? $this->session->data['language'] : $this->config->get('config_language');
        $method_names = $this->settings['payments_widget_language_title'];
        $title        = (isset($method_names[$lang]) ? $method_names[$lang] : '');
        $transaction  = array();


        if ($this->api->verifyMac($request)) {
            $data      = $this->api->extractRequestData($request);
            $reference = false;

            if ( ! empty($data['reference']))
                $transaction = $data;
            elseif ( ! empty($data['transaction']['reference']))
                $transaction = $data['transaction'];

            if ( ! empty($transaction))
                $reference = $transaction['reference'];
        } elseif ( ! empty($request['paymentToken']) && ! empty($request['transaction'])) {
            $request_transaction = $this->api->getTransaction($request['transaction']);

            if ( ! empty($request_transaction->reference)) {
                $reference = $request_transaction->reference;
                $transaction = (array) $request_transaction;
            }
        }

        if ( ! $reference) { $this->response->redirect($this->url->link('common/home')); }

        $order = $this->model_checkout_order->getOrder($reference);

        $this->model_extension_payment_make_commerce->setOrderPaymentMethod($order['order_id'], $title);

        if ($transaction['status'] == $this->config->get('mk_status_completed')) {
            if ( ! empty($transaction['transaction'])) {
                $comment = "Amount: {$transaction['amount']} {$transaction['currency']}<br />Transaction ID: {$transaction['transaction']}";
            } else {
                $amount = number_format((float)$order['total'], 2, '.', '');
                $comment = "Amount: $amount {$order['currency_code']}<br />Transaction ID: {$transaction['id']}";
            }

            $this->model_checkout_order->addOrderHistory($order['order_id'], 2, $comment, false);

            if ( ! empty($this->settings['omniva_courier_register_shipments_as']))
                $this->registerShipment($order);

            $this->response->redirect($this->url->link('checkout/success'));
        } elseif ($transaction['status'] == $this->config->get('mk_status_cancelled')) {
            $this->model_checkout_order->addOrderHistory($order['order_id'], 7);

            $this->response->redirect($this->url->link('checkout/cart', '', true));  // was checkout/checkout
        }
    }

    /**
     * Gets url of the given bank link.
     *
     * @param  object $transaction
     * @param  array  $payment_methods
     * @param  string $country_iso
     * @param  string $name
     * @return string
     */
    private function getPaymentMethodUrlByName($transaction, $payment_methods, $country_iso, $name)
    {
        $url = '';
        $transaction_id = isset($transaction->id) ? $transaction->id : '';

        foreach ($payment_methods as $banklink) {
            if ($banklink['country_iso_code'] == $country_iso && $banklink['name'] == $name) {
                $url_parts = parse_url($banklink['url']);

                $search = $url_parts['scheme'] . '://' . $url_parts['host'];
                $replace = $this->static_gateway_url;

                $url = str_replace($search, $replace, $banklink['url'] . $transaction_id);
            }
        }

        return $url;
    }

    /**
     * Converts the given array to html data attributes.
     *
     * @param  array  $array
     * @return string
     */
    private function convertArrayToHtmlAttributes($array)
    {
        $result = array();

        foreach ($array as $key => $value)
            $result[] = 'data-' . htmlspecialchars($key) . '=' . '"' . htmlspecialchars($value) . '"';

        return implode(' ', $result);
    }

    /**
     * Registers new shipments at carrier systems.
     *
     * @param  array   $order
     * @return boolean
     */
    private function registerShipment($order)
    {
        $adequate_shipping_codes = array('omniva_parcel', 'omniva_courier_by_make_commerce');

        if (strpos($order['shipping_code'], ':') !== false)
            $shipping_code = explode(':', $order['shipping_code']);
        else
            $shipping_code = explode('.', $order['shipping_code']);

        if ( ! isset($shipping_code[0]) || ! in_array($shipping_code[0], $adequate_shipping_codes))
            return false;

        $shipping_request = array('credentials' => array(), 'orders' => array());
        $shipping_type    = (strpos($shipping_code[0], 'omniva_courier') !== false) ? 'omniva_courier': $shipping_code[0];
        $shipping_company = (strpos($shipping_type, 'omniva') !== false) ? 'omniva': $shipping_type;
        $carrier_uc       = mb_strtoupper($shipping_company);
        $m_service_type;

        if ($shipping_type === 'omniva_parcel')
            $m_service_type = 'PA';
        else if ($shipping_type === 'omniva_courier')
            $m_service_type = $this->settings['omniva_courier_register_shipments_as'];

        $shipping_request['credentials'][$carrier_uc] = array(
            'carrier'  => $carrier_uc,
            'username' => $this->getShippingSetting('service_user', $shipping_type),
            'password' => $this->getShippingSetting('service_password', $shipping_type)
        );

        $sender = array(
            'name'  => $this->getShippingSetting('shop_name', $shipping_type),
            'phone' => $this->getShippingSetting('shop_phone', $shipping_type),
            'email' => $this->getShippingSetting('shop_email', $shipping_type)
        );

        if ($shipping_company == 'omniva') {
            $sender['country']    = $this->getShippingSetting('shop_address_country', $shipping_type);
            $sender['city']       = $this->getShippingSetting('shop_address_city', $shipping_type);
            $sender['street']     = $this->getShippingSetting('shop_address_street', $shipping_type);
            $sender['postalCode'] = $this->getShippingSetting('shop_postal_code', $shipping_type);
        }

        $sr_order = array(
            'carrier'   => $carrier_uc,
            'orderId'   => $order['order_id'],
            'recipient' => array(
                'name'  => ($order['shipping_firstname'] ? $order['shipping_firstname'] . ' ' . $order['shipping_lastname'] : $order['firstname'] . ' ' . $order['lastname'] ),
                'phone' => $order['telephone'],
                'email' => $order['email']),
            'sender'      => $sender,
            'destination' => array(),
        );

        if ($shipping_company == 'omniva') {
            $sr_order['destination'] = array(
                'postalCode' => $order['shipping_postcode'],
                'country'    => $order['shipping_iso_code_2'],
                'county'     => '',
                'city'       => $order['shipping_city'],
                'street'     => $order['shipping_address_1'] . ' ' . $order['shipping_address_2']
            );
        }

        if ( ! empty($order['shipping_code']) && ($shipping_type == 'omniva_parcel')) {
            $shipping_code = explode(':', $order['shipping_code']);

            if (isset($shipping_code[1]))
                $sr_order['destination']['destinationId'] = $shipping_code[1];
        }

        if ( ! empty($m_service_type))
            $sr_order['services'] = array('serviceType' => $m_service_type);

        $shipping_request['orders'][] = $sr_order;
        $shipping_request['credentials'] = array_values($shipping_request['credentials']);

        try {
            $response = $this->api->createShipments($shipping_request);
        } catch (Exception $e) {
            error_log($e->getMessage());
            exit();
        }

        $shipments = ( ! empty($response->shipments)) ? $response->shipments : $response;
        $manifest =  ( ! empty($response->manifests)) ? $response->manifests[0] : false;

        foreach ($shipments as $shipment) {
            $shipment_order_id = (int)$shipment->orderId;

            if ($manifest)
                $this->model_extension_payment_make_commerce->updateOrderData(
                    $order['order_id'],
                    'parcel_machine_manifest',
                    $this->sanitizeTextField($manifest)
                );
            if ( ! empty($shipment->orderId) && ! empty($shipment->shipmentId)) {
                $this->model_extension_payment_make_commerce->updateOrderData(
                    $order['order_id'],
                    'parcel_machine_shipment_id',
                    $this->sanitizeTextField($shipment->shipmentId)
                );

                $this->model_extension_payment_make_commerce->deleteOrderData($order['order_id'], 'parcel_machine_error');
            } else if ( ! empty($shipment->orderId) && ! empty($shipment->barCode)) {
                $this->model_extension_payment_make_commerce->updateOrderData(
                    $order['order_id'],
                    'parcel_machine_shipment_id',
                    $this->sanitizeTextField($shipment->barCode)
                );

                $this->model_extension_payment_make_commerce->deleteOrderData($order['order_id'], '_parcel_machine_error');
            } else if ( ! empty($shipment->orderId) && ! empty($shipment->errorMessage)) {
                $this->model_extension_payment_make_commerce->updateOrderData(
                    $order['order_id'],
                    'parcel_machine_error',
                    $this->sanitizeTextField($shipment->errorMessage)
                );
            }
        }

        return true;
    }

    /**
     * Gets settings for specific shipping method.
     *
     * @param  string $name
     * @param  string $type
     * @return string
     */
    private function getShippingSetting($name, $type)
    {
        if (isset($this->settings[$type . '_' . $name]))
            return $this->settings[$type . '_' . $name];
        else
            return '';
    }

    /**
     * Sanitizes a string from user input or from the database.
     *
     * @param  string $string
     * @return string
     */
    private function sanitizeTextField($string)
    {
        if (strpos($string, '<') !== false) {
            $string = $this->preKsesLessThan($string);
            // This will strip extra whitespace for us.
            $string = $this->stripAllTags($string, true);
        } else {
            $string = trim(preg_replace('/[\r\n\t ]+/', ' ', $string));
        }

        $found = false;

        while (preg_match('/%[a-f0-9]{2}/i', $string, $match)) {
            $string = str_replace($match[0], '', $string);
            $found = true;
        }

        if ($found) {
            // Strip out the whitespace that may now exist after removing the octets.
            $string = trim(preg_replace('/ +/', ' ', $string));
        }

        return $string;

    }

    /**
     * Convert lone less than signs.
     *
     * @param  string $text
     * @return string
     */
    private function preKsesLessThan($text) {
        return preg_replace_callback('%<[^>]*?((?=<)|>|$)%', array($this, 'preKsesLessThanCallback'), $text);
    }

    /**
     * Callback function used by preg_replace.
     *
     * @param  array  $matches
     * @return string
     */
    private function preKsesLessThanCallback($matches) {
        if (false === strpos($matches[0], '>'))
            return esc_html($matches[0]);

        return $matches[0];
    }

    /**
     * Properly strip all HTML tags including script and style.
     *
     * @param  string $string
     * @param  bool   $remove_breaks
     * @return string
     */
    private function stripAllTags($string, $remove_breaks = false) {
        $string = preg_replace('@<(script|style)[^>]*?>.*?</\\1>@si', '', $string);
        $string = strip_tags($string);

        if ($remove_breaks)
            $string = preg_replace('/[\r\n\t ]+/', ' ', $string);

        return trim($string);
    }

    /**
     * Sets up transaction parameters and executes a method that creates a transaction via MakeCommerce API.
     *
     * @param  string $amount
     * @param  string $order_id
     * @param  string $return_url
     * @param  string $customer_ip
     * @param  string $billing_country
     * @param  string $locale
     * @return array|object
     */
    private function createTransaction($amount, $order_id, $return_url, $customer_ip, $billing_country, $locale)
    {
        $request_body = array(
            'transaction' => array(
                'amount'    => $amount,
                'currency'  => 'EUR',
                'reference' => $order_id,
                'transaction_url' => array(
                        'return_url' => array(
                            'url'    => $return_url,
                            'method' => 'POST',
                        ),
                        'cancel_url' => array(
                            'url'    => $return_url,
                            'method' => 'POST',
                        ),
                        'notification_url' => array(
                            'url'    => $return_url,
                            'method' => 'POST',
                        ),
                    ),
                ),
            'customer' => array(
                'ip'      => $customer_ip,
                'country' => $billing_country,
                'locale'  => $locale,
            )
        );

        return $this->api->createTransaction($request_body);
    }

    /**
     * Strip slashes from multi-dimensional array.
     *
     * @param  array $value
     * @return array
     */
    private function stripslashesDeep($value)
    {
        $value = is_array($value) ?
                    array_map(array($this, 'stripslashesDeep'), $value) :
                    stripslashes($value);

        return $value;
    }

    /**
     * Checks whether the current url ir HTTPS or not.
     *
     * @param  array $value
     * @return array
     */
    private function isCurrentUrlHttps()
    {
       return isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == "on";
    }
}
