<?php

require_once DIR_SYSTEM . 'library/make_commerce/maksekeskus/vendor/autoload.php';
require_once DIR_SYSTEM . '../admin/model/extension/module/make_commerce.php';

use Maksekeskus\Maksekeskus;

class ModelExtensionShippingOmnivaCourierByMakeCommerce extends Model {
    
    /**
     * Admin model
     */
    private $make_commerce_model;

    /**
     * Module API settings
     */
    private $settings;
    
    /**
     * Model's constructor.
     *
     * @return void                 
     */
    public function __construct($registry)
    {
        parent::__construct($registry);
        $this->config->load('make_commerce');

        $this->make_commerce_model = new ModelExtensionModuleMakeCommerce($registry);

        if ( ! $this->make_commerce_model->isSettingTableCreated())
            return;

        $this->settings    = $this->make_commerce_model->getSettings();
        $this->is_test_env = $this->settings['api_environment'] !== 'live';

        $shop_id     = ($this->is_test_env) ? $this->settings['shop_id_test'] : $this->settings['shop_id_live'];
        $public_key  = ($this->is_test_env) ? $this->settings['public_key_test'] : $this->settings['public_key_live'];
        $private_key = ($this->is_test_env) ? $this->settings['private_key_test'] : $this->settings['private_key_live'];

        $this->api = new Maksekeskus($shop_id, $public_key, $private_key, $this->is_test_env);
    }

    /**
     * Gets called on the checkout page to display MakeCommerce
     * payment option.
     *
     * @param  array $address
     * @return array                 
     */
	public function getQuote($address)
	{
		$this->load->language('extension/shipping/omniva_courier_by_make_commerce');

        $geo_zone_ids              = $this->getGeoZoneIds($address['country_id'], $address['zone_id']);
        $is_within_weight_limit    = $this->isWithinWeigtLimit();
        $is_in_geo_zone            = $this->isAllowedInGeoZone($this->settings['omniva_courier_geo_zone_data'], $geo_zone_ids);;

        if ((float) $this->settings['omniva_courier_maximum_weight'] != 0 && ! $is_within_weight_limit)
            return array();

        if ( ! $is_in_geo_zone)
            return array();
        
        $lang         = (isset($this->session->data['language'])) ? $this->session->data['language'] : $this->config->get('config_language');
        $currency     = $this->session->data['currency'];
        $cost         = $this->currency->convert($this->getShippingCost($geo_zone_ids), 'EUR', $currency);
        $method_names = $this->settings['omniva_courier_method_name_languages'];
        $title        = (isset($method_names[$lang]['method_name']) ? $method_names[$lang]['method_name'] : '');
        $sub_title    = (isset($method_names[$lang]['method_sub_name']) ? $method_names[$lang]['method_sub_name'] : '');

        $quote_data['omniva_courier'] = array(
            'code'         => 'omniva_courier_by_make_commerce.omniva_courier',
            'title'        => $sub_title,
            'cost'         => $this->currency->convert($cost, $currency, 'EUR'),
            'tax_class_id' => $this->settings['omniva_courier_tax_class'],
            'text'         => $this->currency->format($this->tax->calculate($cost, $this->settings['omniva_courier_tax_class'], $this->config->get('config_tax')), $this->session->data['currency'], 1.0000000),
        );

		$method_data = array(
			'code'       => 'omniva_courier',
			'title'      => $title,
            'quote'      => $quote_data,
            'sort_order' => $this->settings['omniva_courier_sort_order'],
            'error'      => '',
		);

		return $method_data;
	}

    /**
     * Gets an array of MakeCommerce settings by calling admin's 
     * model (admin/model/extension/module/make_commerce.php) method getSettings.
     *
     * @return array                 
     */
    public function getSettings()
    {
        return $this->make_commerce_model->getSettings();
    }

    /**
     * Calculates shipping cost based on the settings.
     *
     * @param  array $geo_zone_ids
     * @return array                 
     */
    public function getShippingCost($geo_zone_ids)
    {   
        $are_all_cart_products_free_shipping = $this->areAllCartProductsFreeShipping();

        if ($are_all_cart_products_free_shipping)
            return 0.00;

        $cost       = 20.00; // Default cost if something wents wrong
        $cart_total = $this->cart->getTotal();
        $currency   = $this->session->data['currency'];

        $pricing_type = $this->settings['omniva_courier_pricing_type'];

        if ($pricing_type == 'fixed') {
            // Check if cart amount exceeds free amount
            $cost = $this->settings['omniva_courier_shipping_cost'];
            $free_shipping_amount = (float) $this->settings['omniva_courier_fixed_free_shipping_amount'];           
            $free_shipping_amount = $this->currency->convert($free_shipping_amount, 'EUR', $currency);

            if ($free_shipping_amount != 0 && $cart_total >= $free_shipping_amount)
                $cost = 0.00;
        } else {
            $highest_geo_zone_cost = 0;
            $highest_free_shipping_amount = 0;

            $temp_cost = 0;
            $temp_free_shipping_amount = 0;

            foreach ($geo_zone_ids as $geo_zone_id) {
                if (isset($this->settings['omniva_courier_geo_zone_data'][$geo_zone_id]) && 
                    $this->isAllowedInGeoZone($this->settings['omniva_courier_geo_zone_data'], array($geo_zone_id))) {
                    
                    $temp_cost = $this->settings['omniva_courier_geo_zone_data'][$geo_zone_id]['omniva_courier_cost'];
                    $temp_free_shipping_amount = $this->settings['omniva_courier_geo_zone_data'][$geo_zone_id]['omniva_courier_free_shipping_amount'];

                    if ($temp_cost > $highest_geo_zone_cost)
                        $highest_geo_zone_cost = $temp_cost;

                    if ($temp_free_shipping_amount > $highest_free_shipping_amount)
                        $highest_free_shipping_amount = $temp_free_shipping_amount;
                }
            }

            $cost = $highest_geo_zone_cost;
            $free_geo_zone_shipping_amount = $highest_free_shipping_amount;
            $free_geo_zone_shipping_amount = $this->currency->convert($free_geo_zone_shipping_amount, 'EUR', $currency);

            if ($free_geo_zone_shipping_amount != 0 && $cart_total >= $free_geo_zone_shipping_amount)
                $cost = 0.00;
        }

        return number_format($cost, 2, '.', '');
    }

    /**
     * Gets a list of geo zone ids where the given country and zone are in.
     *
     * @param  int   $address_country_id
     * @param  int   $address_zone_id
     * @return array                 
     */
    public function getGeoZoneIds($address_country_id, $address_zone_id)
    {
        $geo_zone_ids = array();

        $query_obj = $this->db->query(
            "SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE country_id = '" . (int)$address_country_id . "' AND (zone_id = '" . (int)$address_zone_id . "' OR zone_id = '0')");

        if ( ! $query_obj->num_rows)
            return $geo_zone_ids;

        foreach ($query_obj->rows as $row) 
            $geo_zone_ids[] = $row['geo_zone_id'];
        
        return $geo_zone_ids;
    }

    /**
     * Checks if all of the current cart's products are set to be free 
     * from shipping costs
     *
     * @return boolean                 
     */
    private function areAllCartProductsFreeShipping() 
    {
        $result = true;

        $this->load->model('catalog/product');  

        $products = $this->cart->getProducts();

        foreach ($products as $product) {
           $product_info = $this->model_catalog_product->getProduct($product['product_id']);  

           if ( ! isset($product_info['makecommerce_no_shipping_cost']) || $product_info['makecommerce_no_shipping_cost'] !== 'on')
                $result = false;
        }

        return $result;
    }

    /**
     * Checks if the omniva parcel shipping is set to be 
     * availble in the given geo zones.
     *
     * @param  array   $omniva_geo_zone_data
     * @param  array   $geo_zone_ids
     * @return boolean                 
     */
    private function isAllowedInGeoZone($omniva_geo_zone_data, $geo_zone_ids)
    {
        $is_allowed = false;

        if (empty($geo_zone_ids) || empty($omniva_geo_zone_data))
            return $is_allowed;

        foreach ($geo_zone_ids as $geo_zone_id) {
            if (array_key_exists($geo_zone_id, $omniva_geo_zone_data) && 
                $omniva_geo_zone_data[$geo_zone_id]['omniva_courier_geo_zone_active'] == 'on')
                $is_allowed = true;
        }

        return $is_allowed;
    }

    /**
     * Checks if the current cart total weight is within omniva parcel weight limit.
     *
     * @return boolean                 
     */
    private function isWithinWeigtLimit()
    {
        $cart_total_weight = (float) $this->cart->getWeight();
        $weight_limit = (float) $this->settings['omniva_courier_maximum_weight'];

        return $cart_total_weight <= $weight_limit;        
    }

}
